/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "xtprocessor.h"

#include "audioengine.h"

namespace bipscript {

namespace audio {
namespace binding {
audio::Sink *resolve_sink(SQUserPointer obj) { return XtAudioProcessorMap::instance().find(obj); }
audio::Source *resolve_source(SQUserPointer obj) {
    return XtAudioProcessorMap::instance().find(obj);
}
} // namespace binding
} // namespace audio

XtAudioProcessor::XtAudioProcessor(void *key, const char *name, uint32_t inputs, uint32_t outputs,
                                   BXTAudioProcess func)
    : audio::Sink(inputs), audio::Source(outputs), mprocessfunc(func), mkey(key) {
    inputaudio = new float *[inputs];
    StatusServer &statusserver = StatusServer::instance();
    if (statusserver.active()) {
        statusserver.newProcessor(this, name, inputs, outputs);
        reportConnections(this);
        reportConnectors(this);
    }
}

bool XtAudioProcessor::doProcess(samples_t time) {
    // process audio inputs
    samples_t nframes = Clock::period();
    for (uint32_t i = 0; i < audio::Sink::channels(); i++) {
        inputaudio[i] = Sink::inputaudio(i, time, nframes);
    }
    bool active = mprocessfunc(mkey, nframes);
    reportLevels(this, time);
    return audio::Sink::activeEvents() || active;
}

void XtAudioProcessorMap::add(void *key, const char *name, uint32_t inputs, uint32_t outputs,
                              BXTAudioProcess func) {
    AudioEngine &engine = AudioEngine::instance();
    XtAudioProcessor *processor = new XtAudioProcessor(key, name, inputs, outputs, func);
    engine.addProcessor(processor);
    mmap[key] = processor;
}

void XtAudioProcessorMap::reset(bool restart) {
    if (mmap.size()) {
        AudioEngine &engine = AudioEngine::instance();
        StatusServer &statusserver = StatusServer::instance();
        for (auto const &entry : mmap) {
            XtAudioProcessor *processor = entry.second;
            engine.removeProcessor(processor);
            if (restart && statusserver.active()) {
                statusserver.removeProcessor(processor);
            }
        }
        mmap.clear();
    }
}

} // namespace bipscript
