/*
 * This file is part of Bipscript.
 *
 * Bipscript is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef XTPROCESSOR_H
#define XTPROCESSOR_H

#include "audiosink.h"
#include "bipxtapi.h"
#include "objectcache.h"
#include "processorthread.h"

namespace bipscript {

class XtProcessor : public ProcessorThread {
    BXTProcess mprocess;
    BXTProcess mrestart;
    void *marg;

    const static int DEF_PERIOD = 5;

  public:
    XtProcessor(BXTBoolean resetproof, BXTProcess process, BXTProcess restart, void *arg)
        : ProcessorThread(resetproof, DEF_PERIOD), mprocess(process), mrestart(restart), marg(arg) {
        start();
    }
    bool process() override { return mprocess(marg); }
    bool restarted() override { return mrestart(marg); }
};

struct XtMessage {
    void *mdata;
    XtMessage(void *data) : mdata(data) {}
};

typedef TimelineSet<XtMessage> XtTimelineSet;

class XtAudioProcessor : public audio::Sink, public audio::Source {
    BXTAudioProcess mprocessfunc;
    void *mkey;
    float **inputaudio; // pointers to input buffers

  public:
    XtAudioProcessor(void *key, const char *name, uint32_t inputs, uint32_t outputs,
                     BXTAudioProcess func);
    float *audioinputbuffer(int i) { return inputaudio[i]; }
    // processor api
    bool doProcess(samples_t time) override;
    void restart() override {}
    bool connectsTo(AbstractSink *sink) override {
        return sink == this || audio::Sink::connectsTo(sink);
    }
};

class XtAudioProcessorMap : public ObjectCache {
    std::map<void *, XtAudioProcessor *> mmap;
    XtAudioProcessorMap() {}
    virtual ~XtAudioProcessorMap() {}

  public:
    static XtAudioProcessorMap &instance() {
        static XtAudioProcessorMap instance;
        return instance;
    }
    XtAudioProcessor *find(void *key) { return mmap.find(key) != mmap.end() ? mmap[key] : nullptr; }
    void add(void *key, const char *name, uint32_t inputs, uint32_t outputs, BXTAudioProcess func);
    void scriptComplete() override {}
    void reset(bool restart) override;
};

} // namespace bipscript

#endif // XTPROCESSOR_H
